@@ -28,12 +28,17 @@ name    atoms   energy  forces  number_of_atoms
 Fe_bcc  ...
 """
 
+from typing import Callable, Dict, Any, Optional
+
 from warnings import catch_warnings
 
 import numpy as np
 import pandas as pd
 import seaborn as sns
 import matplotlib.pyplot as plt
+
+from ase.atoms import Atoms as ASEAtoms
+
 from pyiron_contrib.atomistics.atomistics.job.structurestorage import StructureStorage
 from pyiron_atomistics.atomistics.structure.atoms import Atoms
 from pyiron_atomistics.atomistics.structure.has_structure import HasStructure
@@ -52,10 +57,10 @@ class TrainingContainer(GenericJob, HasStructure):
         self.__hdf_version__ = "0.3.0"
         self._container = TrainingStorage()
 
-        self.input = DataContainer({
-                "save_neighbors": True,
-                "num_neighbors": 12
-            }, table_name="parameters")
+        self.input = DataContainer(
+            {"save_neighbors": True, "num_neighbors": 12},
+            table_name="parameters"
+        )
 
     def include_job(self, job, iteration_step=-1):
         """
@@ -63,12 +68,39 @@ class TrainingContainer(GenericJob, HasStructure):
 
         Args:
             job (:class:`.AtomisticGenericJob`): job to take structure from
-            iteration_step (int, optional): if job has multiple steps, this selects which to add
+            iteration_step (int, optional): if job has multiple steps, this
+            selects which to add
         """
         self._container.include_job(job, iteration_step)
 
-    @deprecate("Use add_structure instead")
-    def include_structure(self, structure, energy, forces=None, stress=None, name=None):
+    def include_structure(
+        self,
+        structure,
+        energy=None,
+        name=None,
+        **properties
+    ):
+        """
+        Add new structure to structure list and save energy and forces with it.
+
+        For consistency with the rest of pyiron, energy should be in units of eV
+        and forces in eV/A, but no conversion is performed.
+
+        Args:
+            structure_or_job (:class:`~.Atoms`): structure to add
+            energy (float): energy of the whole structure
+            forces (Nx3 array of float, optional): per atom forces, where N is
+                the number of atoms in the structure
+            stress (6 array of float, optional): per structure stresses in voigt
+                notation
+            name (str, optional): name describing the structure
+        """
+        self._container.include_structure(structure, name=name, energy=energy,
+                                          **properties)
+
+    def add_structure(
+        self, structure, energy, forces=None, stress=None, name=None, **arrays
+    ):
         """
         Add new structure to structure list and save energy and forces with it.
 
@@ -82,7 +114,9 @@ class TrainingContainer(GenericJob, HasStructure):
             stress (6 array of float, optional): per structure stresses in voigt notation
             name (str, optional): name describing the structure
         """
-        self._container.include_structure(structure, energy, forces, stress, name)
+        self._container.add_structure(
+            structure, energy, name=name, forces=forces, stress=stress, **arrays
+        )
 
     def include_dataset(self, dataset):
         """
@@ -118,8 +152,11 @@ class TrainingContainer(GenericJob, HasStructure):
         """
         if num_neighbors is None:
             num_neighbors = self.input.num_neighbors
-        n = NeighborsTrajectory(has_structure=self, store=self._container if self.input.save_neighbors else None,
-                                num_neighbors=num_neighbors)
+        n = NeighborsTrajectory(
+            has_structure=self,
+            store=self._container if self.input.save_neighbors else None,
+            num_neighbors=num_neighbors,
+        )
         n.compute_neighbors()
         return n
 
@@ -161,6 +198,9 @@ class TrainingContainer(GenericJob, HasStructure):
         """
         return self._container.to_list(filter_function)
 
+    def to_dict(self):
+        return self._container.to_dict()
+
     def write_input(self):
         pass
 
@@ -187,7 +227,9 @@ class TrainingContainer(GenericJob, HasStructure):
         super().from_hdf(hdf=hdf, group_name=group_name)
         hdf_version = self.project_hdf5.get("HDF_VERSION", "0.1.0")
         if hdf_version == "0.1.0":
-            table = pd.read_hdf(self.project_hdf5.file_name, self.name + "/output/structure_table")
+            table = pd.read_hdf(
+                self.project_hdf5.file_name, self.name + "/output/structure_table"
+            )
             self.include_dataset(table)
         else:
             self._container = TrainingStorage()
@@ -195,6 +237,33 @@ class TrainingContainer(GenericJob, HasStructure):
             if hdf_version == "0.3.0":
                 self.input.from_hdf(self.project_hdf5, "parameters")
 
+    def sample(
+        self, name: str, selector: Callable[[StructureStorage, int], bool]
+    ) -> "TrainingContainer":
+        """
+        Create a new TrainingContainer with structures filtered by selector.
+
+        `self` must have status `finished`.  `selector` is passed the underlying :class:`StructureStorage` of this
+        container and the index of the structure and return a boolean whether to include the structure in the new
+        container or not.  The new container is saved and run.
+
+        Args:
+            name (str): name of the new TrainingContainer
+            selector (Callable[[StructureStorage, int], bool]): callable that selects structure to include
+
+        Returns:
+            :class:`.TrainingContainer`: new container with selected structures
+
+        Raises:
+            ValueError: if a job with the given `name` already exists.
+        """
+        cont = self.project.create.job.TrainingContainer(name)
+        if not cont.status.initialized:
+            raise ValueError(f"Job '{name}' already exists with status: {cont.status}!")
+        cont._container = self._container.sample(selector)
+        cont.run()
+        return cont
+
     @property
     def plot(self):
         """
@@ -202,6 +271,21 @@ class TrainingContainer(GenericJob, HasStructure):
         """
         return TrainingPlots(self._container)
 
+    def iter(self, *arrays, wrap_atoms=True):
+        """
+        Iterate over all structures in this object and all arrays that are defined
+
+        Args:
+            wrap_atoms (bool): True if the atoms are to be wrapped back into the unit cell; passed to
+                               :meth:`.get_structure()`
+            *arrays (str): name of arrays that should be iterated over
+
+        Yields:
+            :class:`pyiron_atomistics.atomistitcs.structure.atoms.Atoms`, arrays: every structure attached to the object and queried arrays
+        """
+        yield from self._container.iter(*arrays, wrap_atoms=wrap_atoms)
+
+
 class TrainingPlots:
     """
     Simple interface to plot various properties of the structures inside the given :class:`.TrainingContainer`.
@@ -222,8 +306,9 @@ class TrainingPlots:
         Returns:
             DataFrame: contains columns 'crystal_system' (str) and 'space_group' (int) for each structure
         """
+
         def get_crystal_system(num):
-            if num in range(1,3):
+            if num in range(1, 3):
                 return "triclinic"
             elif num in range(3, 16):
                 return "monoclinic"
@@ -240,17 +325,20 @@ class TrainingPlots:
 
         def extract(s):
             spg = s.get_symmetry(symprec=symprec).spacegroup["Number"]
-            return {'space_group': spg, 'crystal_system': get_crystal_system(spg)}
+            return {"space_group": spg, "crystal_system": get_crystal_system(spg)}
 
         return pd.DataFrame(map(extract, self._train.iter_structures()))
 
-    def cell(self):
+    def cell(self, angle_in_degrees=True):
         """
         Plot histograms of cell parameters.
 
         Plotted are atomic volume, density, cell vector lengths and cell vector angles in separate subplots all on a
         log-scale.
 
+        Args:
+            angle_in_degrees (bool): whether unit for angles is degree or radians
+
         Returns:
             `DataFrame`: contains the plotted information in the columns:
                             - a: length of first vector
@@ -266,30 +354,38 @@ class TrainingPlots:
         C = self._train.get_array("cell")
 
         def get_angle(cell, idx=0):
-            return np.arccos(np.dot(cell[idx], cell[(idx+1)%3]) \
-                    / np.linalg.norm(cell[idx]) / np.linalg.norm(cell[(idx+1)%3]))
+            return np.arccos(
+                np.dot(cell[idx], cell[(idx + 1) % 3])
+                / np.linalg.norm(cell[idx])
+                / np.linalg.norm(cell[(idx + 1) % 3])
+            )
 
         def extract(n, c):
             return {
-                    "a": np.linalg.norm(c[0]),
-                    "b": np.linalg.norm(c[1]),
-                    "c": np.linalg.norm(c[2]),
-                    "alpha": get_angle(c, 0),
-                    "beta": get_angle(c, 1),
-                    "gamma": get_angle(c, 2),
+                "a": np.linalg.norm(c[0]),
+                "b": np.linalg.norm(c[1]),
+                "c": np.linalg.norm(c[2]),
+                "alpha": get_angle(c, 0),
+                "beta": get_angle(c, 1),
+                "gamma": get_angle(c, 2),
             }
+
         df = pd.DataFrame([extract(n, c) for n, c in zip(N, C)])
         df["V"] = np.linalg.det(C)
         df["N"] = N
+        if angle_in_degrees:
+            df["alpha"] = np.rad2deg(df["alpha"])
+            df["beta"] = np.rad2deg(df["beta"])
+            df["gamma"] = np.rad2deg(df["gamma"])
 
         plt.subplot(1, 4, 1)
         plt.title("Atomic Volume")
-        plt.hist(df.V/df.N, bins=20, log=True)
+        plt.hist(df.V / df.N, bins=20, log=True)
         plt.xlabel(r"$V$ [$\AA^3$]")
 
         plt.subplot(1, 4, 2)
         plt.title("Density")
-        plt.hist(df.N/df.V, bins=20, log=True)
+        plt.hist(df.N / df.V, bins=20, log=True)
         plt.xlabel(r"$\rho$ [$\AA^{-3}$]")
 
         plt.subplot(1, 4, 3)
@@ -300,7 +396,11 @@ class TrainingPlots:
         plt.subplot(1, 4, 4)
         plt.title("Lattice Vector Angles")
         plt.hist([df.alpha, df.beta, df.gamma], log=True)
-        plt.xlabel(r"$\alpha,\beta,\gamma$")
+        if angle_in_degrees:
+            label = r"$\alpha,\beta,\gamma$ [°]"
+        else:
+            label = r"$\alpha,\beta,\gamma$ [rad]"
+        plt.xlabel(label)
 
         return df
 
@@ -387,14 +487,21 @@ class TrainingPlots:
         """
         if not self._train.has_array("shells"):
             raise ValueError(
-                    "TrainingContainer contains no neighbor information, call TrainingContainer.get_neighbors first!"
+                "TrainingContainer contains no neighbor information, call TrainingContainer.get_neighbors first!"
             )
-        shells = self._train.get_array('shells')
-        shell_index = shells[np.newaxis, :, :] == np.arange(1, num_shells+1)[:, np.newaxis, np.newaxis]
+        shells = self._train.get_array("shells")
+        shell_index = (
+            shells[np.newaxis, :, :]
+            == np.arange(1, num_shells + 1)[:, np.newaxis, np.newaxis]
+        )
         neigh_count = shell_index.sum(axis=-1)
-        ticks = np.arange(neigh_count.min(), neigh_count.max()+1)
-        plt.hist(neigh_count.T, bins=ticks-0.5,
-                 log=True, label=[f"{i}." for i in range(1, num_shells+1)])
+        ticks = np.arange(neigh_count.min(), neigh_count.max() + 1)
+        plt.hist(
+            neigh_count.T,
+            bins=ticks - 0.5,
+            log=True,
+            label=[f"{i}." for i in range(1, num_shells + 1)],
+        )
         plt.xticks(ticks)
         plt.xlabel("Number of Neighbors")
         plt.legend(title="Shell")
@@ -407,45 +514,50 @@ class TrainingPlots:
         Args:
             num_shells (int): maximum shell to plot
         """
-        if not self._train.has_array("shells") or not self._train.has_array("distances"):
+        if not self._train.has_array("shells") or not self._train.has_array(
+            "distances"
+        ):
             raise ValueError(
-                    "TrainingContainer contains no neighbor information, call TrainingContainer.get_neighbors first!"
+                "TrainingContainer contains no neighbor information, call TrainingContainer.get_neighbors first!"
             )
-        dists = self._train.get_array('distances')
+        dists = self._train.get_array("distances")
         R = dists.flatten()
-        shells = self._train.get_array('shells')
+        shells = self._train.get_array("shells")
         S = shells.ravel()
-        d = pd.DataFrame({"distance": R[S < num_shells + 1], "shells": S[S < num_shells + 1]})
-        sns.violinplot(y=d.shells, x=d.distance, scale='width', orient='h')
+        d = pd.DataFrame(
+            {"distance": R[S < num_shells + 1], "shells": S[S < num_shells + 1]}
+        )
+        sns.violinplot(y=d.shells, x=d.distance, scale="width", orient="h")
         plt.xlabel(r"Distance [$\AA$]")
         plt.ylabel("Shell")
 
+    def forces(self, axis: Optional[int] = None):
+        """
+        Plot a histogram of all forces.
+
+        Args:
+            axis (int, optional): plot only forces along this axis, if not given plot all forces
+        """
+        f = self._train.get_array("forces")
+        if axis is not None:
+            f = f[:, axis]
+        else:
+            f = f.ravel()
+        plt.hist(f, bins=20)
+        plt.xlabel(r"Force [eV/$\mathrm{\AA}$]")
+
+
 class TrainingStorage(StructureStorage):
     def __init__(self):
         super().__init__()
         self.add_array("energy", dtype=np.float64, per="chunk", fill=np.nan)
-        self.add_array("forces", shape=(3,), dtype=np.float64, per="element", fill=np.nan)
+        self.add_array("forces", shape=(3,), dtype=np.float64, per="element",
+                       fill=np.nan)
         # save stress in voigt notation
-        self.add_array("stress", shape=(6,), dtype=np.float64, per="chunk", fill=np.nan)
+        self.add_array("stress", shape=(6,), dtype=np.float64, per="chunk",
+                       fill=np.nan)
         self._table_cache = None
-
-    @property
-    def _table(self):
-        if self._table_cache is None or len(self._table_cache) != len(self):
-            self._table_cache = pd.DataFrame({
-                "name":             [self.get_array("identifier", i)
-                                        for i in range(len(self))],
-                "atoms":            [self.get_structure(i)
-                                        for i in range(len(self))],
-                "energy":           [self.get_array("energy", i)
-                                        for i in range(len(self))],
-                "forces":           [self.get_array("forces", i)
-                                        for i in range(len(self))],
-                "stress":           [self.get_array("stress", i)
-                                        for i in range(len(self))],
-            })
-            self._table_cache["number_of_atoms"] = [len(s) for s in self._table_cache.atoms]
-        return self._table_cache
+        self.to_pandas()
 
     def to_pandas(self):
         """
@@ -462,7 +574,20 @@ class TrainingStorage(StructureStorage):
         Returns:
             :class:`pandas.DataFrame`: collected structures
         """
-        return self._table
+        if self._table_cache is None or len(self._table_cache) != len(self):
+            self._table_cache = pd.DataFrame(
+                {
+                    "name": [self.get_array("identifier", i) for i in range(len(self))],
+                    "atoms": [self.get_structure(i) for i in range(len(self))],
+                    "energy": [self.get_array("energy", i) for i in range(len(self))],
+                    "forces": [self.get_array("forces", i) for i in range(len(self))],
+                    "stress": [self.get_array("stress", i) for i in range(len(self))],
+                }
+            )
+            self._table_cache["number_of_atoms"] = [
+                len(s) for s in self._table_cache.atoms
+            ]
+        return self._table_cache
 
     def include_job(self, job, iteration_step=-1):
         """
@@ -490,14 +615,32 @@ class TrainingStorage(StructureStorage):
         if stress is not None:
             stress = np.asarray(stress)
             if stress.shape == (3, 3):
-                stress = np.array([stress[0, 0], stress[1 ,1], stress[2, 2],
-                                   stress[1, 2], stress[0, 2], stress[0, 1]])
-        self.include_structure(job.get_structure(iteration_step=iteration_step),
-                               energy=energy, forces=forces, stress=stress,
-                               name=job.name)
+                stress = np.array(
+                    [
+                        stress[0, 0],
+                        stress[1, 1],
+                        stress[2, 2],
+                        stress[1, 2],
+                        stress[0, 2],
+                        stress[0, 1],
+                    ]
+                )
+        self.include_structure(
+            job.get_structure(iteration_step=iteration_step),
+            energy=energy,
+            forces=forces,
+            stress=stress,
+            name=job.name,
+        )
 
     @deprecate("Use add_structure instead")
-    def include_structure(self, structure, energy, forces=None, stress=None, name=None):
+    def include_structure(
+        self,
+        structure,
+        energy,
+        name=None,
+        **properties
+    ):
         """
         Add new structure to structure list and save energy and forces with it.
 
@@ -511,23 +654,19 @@ class TrainingStorage(StructureStorage):
             stress (6 array of float, optional): per structure stresses in voigt notation
             name (str, optional): name describing the structure
         """
-        self.add_structure(structure, energy, identifier=name, forces=forces, stress=stress)
+        self.add_structure(structure, identifier=name, energy=energy,
+                           **properties)
 
+    def add_structure(
+        self,
+        structure: Atoms,
+        energy,
+        identifier=None,
+        **arrays
+    ) -> None:
+        super().add_structure(structure, identifier=identifier, energy=energy,
+                              **arrays)
 
-    def add_structure(self, structure, energy, identifier=None, forces=None, stress=None, **arrays):
-        data = {"energy": energy}
-        if forces is not None:
-            data["forces"] = forces
-        if stress is not None:
-            data["stress"] = stress
-        super().add_structure(structure, identifier, **data)
-        if self._table_cache:
-            self._table = self._table.append(
-                    {"name": identifier, "atoms": structure, "energy": energy, "forces": forces, "stress": stress,
-                     "number_of_atoms": len(structure)},
-                    ignore_index=True)
-
-        
     def include_dataset(self, dataset):
         """
         Add a pandas DataFrame to the saved structures.
@@ -537,12 +676,26 @@ class TrainingStorage(StructureStorage):
             - atoms(:class:`ase.Atoms`): the atomic structure
             - energy(float): energy of the whole structure
             - forces (Nx3 array of float): per atom forces, where N is the number of atoms in the structure
+            - charges (Nx3 array of floats):
             - stress (6 array of float): per structure stress in voigt notation
         """
-        self._table_cache = self._table.append(dataset, ignore_index=True)
-        # in case given dataset has more columns than the necessary ones, swallow/ignore them in *_
-        for name, atoms, energy, forces, stress, *_ in dataset.itertuples(index=False):
-            self.add_structure(atoms, name, energy=energy, forces=forces, stress=stress)
+        if (
+            "name" not in dataset.columns
+            or "atoms" not in dataset.columns
+            or "energy" not in dataset.columns
+        ):
+            raise ValueError(
+                "At least columns 'name', 'atoms' and 'energy' must be present in dataset!"
+            )
+        for row in dataset.itertuples(index=False):
+            kwargs = {}
+            if hasattr(row, "forces"):
+                kwargs["forces"] = row.forces
+            if hasattr(row, "stress"):
+                kwargs["stress"] = row.stress
+            self.add_structure(
+                row.atoms, energy=row.energy, identifier=row.name, **kwargs
+            )
 
     def to_list(self, filter_function=None):
         """
@@ -554,12 +707,53 @@ class TrainingStorage(StructureStorage):
         Returns:
             tuple: list of structures, energies, forces, and the number of atoms
         """
-        if filter_function is None:
-            data_table = self._table
-        else:
-            data_table = filter_function(self._table)
+        data_table = self.to_pandas()
+        if filter_function is not None:
+            data_table = filter_function(data_table)
         structure_list = data_table.atoms.to_list()
         energy_list = data_table.energy.to_list()
         force_list = data_table.forces.to_list()
         num_atoms_list = data_table.number_of_atoms.to_list()
-        return structure_list, energy_list, force_list, num_atoms_list
+
+        return (structure_list, energy_list, force_list, num_atoms_list)
+
+    def to_dict(self) -> Dict[str, Any]:
+        """Return a dictionary of all structures and training properties."""
+        dict_arrays = {}
+
+        # Get structure information.
+        dict_arrays['structure'] = list(self.iter_structures())
+
+        # Some arrays are only for internal usage or structure information that
+        # was already saved in dict['structure'].
+        internal_arrays = ['start_index', 'length', 'cell', 'pbc', 'positions',
+                           'symbols']
+        for array in self.list_arrays():
+            # Skip internal arrays.
+            if array in internal_arrays:
+                continue
+
+            dict_arrays[array] = self.get_array_ragged(array)
+        return dict_arrays
+
+    def iter(self, *arrays, wrap_atoms=True):
+        """
+        Iterate over all structures in this object and all arrays that are defined
+
+        Args:
+            wrap_atoms (bool): True if the atoms are to be wrapped back into the unit cell; passed to
+                               :meth:`.get_structure()`
+            *arrays (str): name of arrays that should be iterated over
+
+        Yields:
+            :class:`pyiron_atomistics.atomistitcs.structure.atoms.Atoms`, arrays: every structure attached to the object and queried arrays
+        """
+        array_vals = (self.get_array_ragged(a) for a in arrays)
+        yield from zip(self.iter_structures(), *array_vals)
+
+    @property
+    def plot(self):
+        """
+        :class:`.TrainingPlots`: plotting interface
+        """
+        return TrainingPlots(self)
